#ifndef _CDDATA_H
#define _CDDATA_H

#include <vector>

// The following definitions should come from
//
//#include <ntddcdrm.h>
//
// But that's part of the DDK which most people won't have and is in any case
// harder to get hold of these days.

#ifndef CTL_CODE
#define CTL_CODE( DeviceType, Function, Method, Access ) (                 \
    ((DeviceType) << 16) | ((Access) << 14) | ((Function) << 2) | (Method) \
)
#define METHOD_BUFFERED                 0
#define FILE_DEVICE_CD_ROM              0x00000002
#define FILE_READ_ACCESS          ( 0x0001 )    // file & pipe
#endif

#define IOCTL_CDROM_BASE                  FILE_DEVICE_CD_ROM

#define IOCTL_CDROM_READ_TOC \
  CTL_CODE(IOCTL_CDROM_BASE, 0x0000, METHOD_BUFFERED, FILE_READ_ACCESS)
#define IOCTL_CDROM_READ_TOC_EX \
  CTL_CODE(IOCTL_CDROM_BASE, 0x0015, METHOD_BUFFERED, FILE_READ_ACCESS)

#define MAXIMUM_NUMBER_TRACKS             100
#define MAXIMUM_CDROM_SIZE                804
#define MINIMUM_CDROM_READ_TOC_EX_SIZE    2

typedef struct _TRACK_DATA {
  UCHAR  Reserved;
  UCHAR  Control : 4;
  UCHAR  Adr : 4;
  UCHAR  TrackNumber;
  UCHAR  Reserved1;
  UCHAR  Address[4];
} TRACK_DATA, *PTRACK_DATA;

typedef struct _CDROM_TOC {
  UCHAR  Length[2];
  UCHAR  FirstTrack;
  UCHAR  LastTrack;
  TRACK_DATA  TrackData[MAXIMUM_NUMBER_TRACKS];
} CDROM_TOC, *PCDROM_TOC;

#define CDROM_TOC_SIZE sizeof(CDROM_TOC)

// Now our own definitions

typedef struct {
    int idx;
    CString path;
    CString devname;
    CString name;
    DWORD scsi_port;
    DWORD scsi_bus;
    DWORD scsi_targ;
    DWORD scsi_lun;
} drv_info;

class CCDDrives
{
public:
    CCDDrives();

    int getNumDrives();

    drv_info& getDrive(int i);
    CString getDrivePath(int i);
    CString getDriveDesc(int i);

protected:
    std::vector<drv_info> m_drives;
};

class CCDData
{
public:
    CCDData();

    void Refresh(drv_info &drv);
    bool Eject(drv_info &drv);
    bool isDiskPresent();

    DWORD getWin32Serial();
    DWORD getMciId();
    DWORD getCDDBDISCID();

    int getNumTracks();

    int getPlayTimeFrames();
    void getPlayTimeMSF(UCHAR *d);

    int getTrackStartFrames(int i);
    void getTrackStartMSF(UCHAR *d, int i);
    int getTrackLengthFrames(int i);
    void getTrackLengthMSF(UCHAR *d, int i);

protected:
    bool m_valid;
    CDROM_TOC m_toc;
    DWORD m_dwWin32Serial;
    DWORD m_dwMCIID;
    DWORD m_cddb;
};

#endif /* _CDDATA_H */