// maindlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "resource.h"

#include "maindlg.h"

//
//
//

BOOL CMainDlg::PreTranslateMessage(MSG* pMsg)
{
    return IsDialogMessage(pMsg);
}

BOOL CMainDlg::OnIdle()
{
    return FALSE;
}

LRESULT CMainDlg::OnInitDialog(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
{
    // center the dialog on the screen
    CenterWindow();

    // set icons
    HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
        IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
    SetIcon(hIcon, TRUE);
    HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
        IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
    SetIcon(hIconSmall, FALSE);

    m_hbrRed = CreateSolidBrush(RGB(255, 0, 0));
    m_hbrGreen = CreateSolidBrush(RGB(0, 255, 0));
    m_busy = false;

    SendDlgItemMessage(IDC_COMBO_DRIVES, CB_RESETCONTENT, 0, 0);
    for ( int i=0 ; i<m_drives.getNumDrives() ; i++ ) {
        CString p = m_drives.getDrivePath(i);
        CString d = m_drives.getDriveDesc(i);

        if (d.GetLength()>0) {
            p.TrimRight(_T('\\'));
            p += _T(" ");
            p += d;
        }

        int j = SendDlgItemMessage(IDC_COMBO_DRIVES, CB_ADDSTRING, 0, (LPARAM)(LPCTSTR)p);
        SendDlgItemMessage(IDC_COMBO_DRIVES, CB_SETITEMDATA, j, i);
    }
    SendDlgItemMessage(IDC_COMBO_DRIVES, CB_SETCURSEL, 0, 0);

    // register object for message filtering and idle updates
    CMessageLoop* pLoop = _Module.GetMessageLoop();
    ATLASSERT(pLoop != NULL);
    pLoop->AddMessageFilter(this);
    pLoop->AddIdleHandler(this);

    UIAddChildWindowContainer(m_hWnd);

    return TRUE;
}

void CMainDlg::SetBusy(bool busy)
{
    m_busy = busy;
    ::RedrawWindow(GetDlgItem(IDC_STATIC_STATUS), NULL, NULL, RDW_INVALIDATE|RDW_UPDATENOW);
}

LRESULT CMainDlg::OnOK(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    SetBusy(true);
    Clear();

    int drvn = SendDlgItemMessage(IDC_COMBO_DRIVES, CB_GETCURSEL, 0, 0);
    if (drvn==CB_ERR) {
        Output(_T("No drive selected\r\n"));
        SetBusy(false);
        return 0;
    }
    drvn = SendDlgItemMessage(IDC_COMBO_DRIVES, CB_GETITEMDATA, drvn, 0);

    drv_info &drv = m_drives.getDrive(drvn);
    CString drvp = m_drives.getDrivePath(drvn);
    drvp.MakeUpper();

    Output(_T("Drive %s\r\n\r\n"), (LPCTSTR)drvp);

    m_data.Refresh(drv);

    if (!m_data.isDiskPresent()) {
        Output(_T("No audio CD present\r\n"));
        SetBusy(false);
        return 0;
    }

    Output(_T("Win32 Serial: %d (0x%p)\r\n"), m_data.getWin32Serial(), m_data.getWin32Serial());
    Output(_T("MCI ID: %d (0x%p)\r\n"), m_data.getMciId(), m_data.getMciId());
    Output(_T("CDDB DISCID: %d (0x%p)\r\n"), m_data.getCDDBDISCID(), m_data.getCDDBDISCID());

    UCHAR dlen[4];
    m_data.getPlayTimeMSF(dlen);

    Output(_T("Playing time: %02.2d:%02.2d:%02.2d\r\n\r\n"), dlen[1], dlen[2], dlen[3]);

    for ( int i=1 ; i<=m_data.getNumTracks() ; i++ ) {
        UCHAR st[4], len[4];
        m_data.getTrackStartMSF(st, i);
        m_data.getTrackLengthMSF(len, i);

        Output(_T("Track %02.2d: Start %02.2d:%02.2d:%02.2d Length %02.2d:%02.2d:%02.2d\r\n"),
            i, st[1], st[2], st[3], len[1], len[2], len[3]);
    }

    if (IsDlgButtonChecked(IDC_CHECK_AUTOEJECT)==BST_CHECKED) {
        BOOL dummy=TRUE;
        OnEject(0, 0, NULL, dummy);
    }

    SetBusy(false);
    return 0;
}

LRESULT CMainDlg::OnCancel(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    CloseDialog(wID);
    return 0;
}

void CMainDlg::CloseDialog(int nVal)
{
    DestroyWindow();
    ::PostQuitMessage(nVal);
}

LRESULT CMainDlg::OnCopy(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    if (!m_data.isDiskPresent()) {
        MessageBeep(MB_OK);
        return 0;
    }

    CString buf, t;
    int i, n = m_data.getNumTracks();

    t.Format(_T("N=%d\r\n"), n);
    buf += t;
    UCHAR dlen[4];
    m_data.getPlayTimeMSF(dlen);
    t.Format(_T("T=%02.2d:%02.2d:%02.2d\r\n"), 0, dlen[1], dlen[2]);
    buf += t;

    CString toc;
    for ( i=1 ; i<=n+1 ; i++ ) {
        t.Format("%d", m_data.getTrackStartFrames(i));
        if (toc.GetLength()>0) toc += _T(",");
        toc += t;
    }
    t.Format(_T("TOC=%s\r\n"), toc);
    buf += t;

    t.Format(_T("WIN32=%p\r\n"), m_data.getWin32Serial());
    buf += t;
    t.Format(_T("MCIID=%p\r\n"), m_data.getMciId());
    buf += t;
    t.Format(_T("CDDB=%p\r\n"), m_data.getCDDBDISCID());
    buf += t;

    for ( i=1 ; i<=n ; i++ ) {
        UCHAR len[4];
        m_data.getTrackLengthMSF(len, i);
        t.Format(_T("T%d=%02.2d:%02.2d:%02.2d\r\n"), i, 0, len[1], len[2]);
        buf += t;
    }

    if (OpenClipboard()) {
        EmptyClipboard();

        HGLOBAL h = GlobalAlloc(GHND, sizeof(TCHAR)*(buf.GetLength()+1));
        LPVOID p = GlobalLock(h);
        memcpy(p, (LPCTSTR)buf, sizeof(TCHAR)*(buf.GetLength()+1));
        GlobalUnlock(h);

        SetClipboardData(CF_TEXT, h);

        CloseClipboard();
    }


    return 0;
}

LRESULT CMainDlg::OnEject(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
{
    int drvn = SendDlgItemMessage(IDC_COMBO_DRIVES, CB_GETCURSEL, 0, 0);
    if (drvn==CB_ERR) {
        MessageBeep(MB_OK);
        return 0;
    }
    drvn = SendDlgItemMessage(IDC_COMBO_DRIVES, CB_GETITEMDATA, drvn, 0);

    drv_info &drv = m_drives.getDrive(drvn);
    m_data.Eject(drv);

    return 0;
}
